""" Read GFF files and output isomiRs compatible format"""
from __future__ import print_function

import os

import mirtop.libs.logger as mylog
from mirtop.mirna import fasta, mapper
from mirtop.gff.classgff import feature
from mirtop.gff.header import read_samples
from mirtop.mirna.realign import get_mature_sequence, align_from_variants
from mirtop.mirna.realign import read_id, variant_to_5p, \
                                 variant_to_3p, variant_to_add

logger = mylog.getLogger(__name__)


def convert(args):
    """
    Main function to convert from GFF3 to isomiRs Bioc Package.

    Args:
      *args*: supported options for this sub-command.
        See *mirtop.libs.parse.add_subparser_export()*.
    """
    precursors = fasta.read_precursor(args.hairpin, args.sps)
    matures = mapper.read_gtf_to_precursor(args.gtf)
    for fn in args.files:
        logger.info("Reading %s" % fn)
        _read_file(fn, precursors, matures, args.out)


def _read_file(fn, precursors, matures, out_dir):
    samples = read_samples(fn)
    for sample in samples:
        with open(os.path.join(out_dir, "%s.mirna" % sample), 'w') as outh:
            print("\t".join(
                ["seq", "name", "freq", "mir", "start", "end",
                 "mism", "add", "t5", "t3", "s5", "s3", "DB",
                 "precursor", "ambiguity"]), file=outh)
    with open(fn) as inh:
        for line in inh:
            if line.startswith("#"):
                continue
            gff = feature(line)
            cols = gff.columns
            attr = gff.attributes
            read = read_id(attr["UID"])
            t5 = variant_to_5p(precursors[attr["Parent"]],
                               matures[attr["Parent"]][attr["Name"]],
                               attr["Variant"])
            t3 = variant_to_3p(precursors[attr["Parent"]],
                               matures[attr["Parent"]][attr["Name"]],
                               attr["Variant"])
            add = variant_to_add(read,
                                 attr["Variant"])
            mature_sequence = get_mature_sequence(
                precursors[attr["Parent"]],
                matures[attr["Parent"]][attr["Name"]])
            mm = align_from_variants(read,
                                     mature_sequence,
                                     attr["Variant"])
            if len(mm) > 1:
                continue
            elif len(mm) == 1:
                mm = "".join(map(str, mm[0]))
            else:
                mm = "0"
            hit = attr["Hits"] if "Hits" in attr else "1"
            logger.debug("exporter::isomir::decode %s" % [attr["Variant"],
                                                          t5, t3, add, mm])
            # Error if attr["Read"] doesn't exist
            print(cols)
            line = [read, attr["Read"], "0", attr["Name"],
                    cols['source'], cols['type'],
                    mm, add, t5, t3, "NA", "NA", "miRNA", attr["Parent"], hit]
            for sample, counts in zip(samples, attr["Expression"].split(",")):
                with open(os.path.join(out_dir, "%s.mirna" % sample),
                          'a') as outh:
                    line[2] = counts
                    print("\t".join(line), file=outh)



""" Read seqbuster files"""

from collections import defaultdict

import mirtop.libs.logger as mylog
from mirtop.mirna.realign import isomir, hits
from mirtop.bam import filter

logger = mylog.getLogger(__name__)


def header():
    """
    Custom header for seqbuster importer.

    Returns:
        *(str)*: seqbuster header string.
    """
    h = ("## CMD: seqbuster: http://seqcluster.readthedocs.io/mirna_annotation.html#mirna-isomir-annotation-with-java\n"
        "# iso_snp are not filtered yet. Use isomiRs R pacakge to correct for error sequencing\n")
    return h


def read_file(fn, args):
    """
    Read seqbuster file and convert to mirtop GFF format.

    Args:
        *fn(str)*: file name with seqbuster output information.

        *database(str)*: database name.

        *args(namedtuple)*: arguments from command line.
            See *mirtop.libs.parse.add_subparser_gff()*.

    Returns:
        *reads*: dictionary where keys are read_id and values are *mirtop.realign.hits*

    """
    precursors = args.precursors
    reads = defaultdict(hits)
    with open(fn) as handle:
        handle.readline()
        for line in handle:
            cols = line.strip().split("\t")
            query_name = cols[1]
            query_sequence = cols[0]
            reference_start = int(cols[4]) - 1
            seqbuster_iso = ":".join(cols[6:10])
            if query_name not in reads and query_sequence==None:
                continue
            if query_sequence and query_sequence.find("N") > -1:
                continue
            if query_name not in reads:
                reads[query_name].set_sequence(query_sequence)
                reads[query_name].counts = _get_freq(query_name)
            chrom = cols[13]
            logger.debug("\nSEQBUSTER::NEW::query: {query_sequence}\n"
                         "  precursor {chrom}\n"
                         "  name:  {query_name}\n"
                         "  start: {reference_start}\n"
                         "  iso: {seqbuster_iso}".format(**locals()))
            # logger.debug("SEQBUSTER:: cigar {cigar}".format(**locals()))
            iso = isomir()
            iso.align = line
            iso.set_pos(reference_start, len(reads[query_name].sequence))
            logger.debug("SEQBUSTER:: start %s end %s" % (iso.start, iso.end))
            if len(precursors[chrom]) < reference_start + len(reads[query_name].sequence):
                continue
            iso.subs, iso.add, iso.cigar = filter.tune(reads[query_name].sequence,
                                                       precursors[chrom],
                                                       reference_start, None)
            logger.debug("SEQBUSTER::After tune start %s end %s" % (iso.start, iso.end))
            if len(iso.subs) < 2:
                reads[query_name].set_precursor(chrom, iso)
    logger.info("Hits: %s" % len(reads))
    return reads


def _get_freq(name):
    """
    Check if name read contains counts (_xNumber)
    """
    try:
        counts = int(name.split("_x")[1])
    except:
        return 0
    return counts



current = "1.0"
GFFv = {'1.0': ["iso_5p", "iso_3p", "iso_add", "iso_snp_seed",
                "iso_snp_central_offset", "iso_snp_central",
                "iso_snp_central_supp", "iso_snp", "NA"]}
ATTRv = {'1.0': set(('IDU', 'Name', 'Alias', 'Parent',
                     'Read', 'Variant', 'Changes',
                     'Cigar', 'Hits', 'Genomic',
                     'Expression', 'Filter', 'Cigar',
                     'Seed_fam'))}



"""functions for explore tool"""
from __future__ import print_function

from mirtop.libs.do import run


def get_fasta(bed_file, ref, out_fa):
    """Run bedtools to get fasta from bed file"""
    cmd = "bedtools getfasta -s -fi {ref} -bed {bed_file} -fo {out_fa}"
    run(cmd.format(**locals()))



""" Read bam files"""

import os.path as op
import os
import pysam
from collections import defaultdict

from mirtop.libs import do
from mirtop.libs.utils import file_exists
import mirtop.libs.logger as mylog
from mirtop.mirna.realign import isomir, hits
from mirtop.bam import filter

logger = mylog.getLogger(__name__)


def read_bam(bam_fn, args, clean=True):
    """
    Read bam file and perform realignment of hits

    Args:
        *bam_fn*: a BAM file with alignments to the precursor

        *precursors*: dict with keys being precursor names and values
            being sequences. Come from mirtop.mirna.fasta.read_precursor().

        *clean*: Use mirtop.filter.clean_hits() to remove lower score hits.

    Returns:
        *reads (dict)*:
             keys are read_id and values are *mirtop.realign.hits*

    """
    precursors = args.precursors
    bam_fn = _sam_to_bam(bam_fn)
    bam_fn = _bam_sort(bam_fn)
    mode = "r" if bam_fn.endswith("sam") else "rb"
    handle = pysam.Samfile(bam_fn, mode)
    reads = defaultdict(hits)
    for line in handle:
        if line.reference_id < 0:
            logger.debug("READ::Sequence not mapped: %s" % line.reference_id)
            continue
        query_name = line.query_name
        if query_name not in reads and not line.query_sequence:
             continue
        logger.debug(("READ::Read name:{0} and Read sequence:{1}").format(line.query_name, line.query_sequence))
        if line.query_sequence and line.query_sequence.find("N") > -1:
            continue
        if query_name not in reads:
            reads[query_name].set_sequence(line.query_sequence)
            reads[query_name].counts = _get_freq(query_name)
        if line.is_reverse:
            logger.debug("READ::Sequence is reverse: %s" % line.query_name)
            continue
        chrom = handle.getrname(line.reference_id)
        cigar = line.cigartuples
        iso = isomir()
        iso.align = line
        iso.set_pos(line.reference_start, len(reads[query_name].sequence))
        logger.debug("READ::From BAM start %s end %s at chrom %s" % (iso.start, iso.end, chrom))
        if len(precursors[chrom]) < line.reference_start + len(reads[query_name].sequence):
            logger.debug("READ::%s start + %s sequence size are bigger than"
                         " size precursor %s" % (
                                                 line.reference_id,
                                                 len(reads[query_name].sequence),
                                                 len(precursors[chrom])))
            continue
        iso.subs, iso.add, iso.cigar = filter.tune(
            reads[query_name].sequence, precursors[chrom],
            line.reference_start, cigar)
        logger.debug("READ::After tune start %s end %s" % (iso.start, iso.end))
        logger.debug("READ::iso add %s iso subs %s" % (iso.add, iso.subs))

        if len(iso.subs) < 2:
            reads[query_name].set_precursor(chrom, iso)
    logger.info("Hits: %s" % len(reads))
    if clean:
        reads = filter.clean_hits(reads)
        logger.info("Hits after clean: %s" % len(reads))
    return reads


def _sam_to_bam(bam_fn):
    if not bam_fn.endswith("bam"):
        bam_out = "%s.bam" % os.path.splitext(bam_fn)[0]
        cmd = "samtools view -Sbh {bam_fn} -o {bam_out}"
        do.run(cmd.format(**locals()))
        return bam_out
    return bam_fn


def _bam_sort(bam_fn):
    bam_sort_by_n = op.splitext(bam_fn)[0] + "_sort.bam"
    if not file_exists(bam_sort_by_n):
        do.run(("samtools sort -n -o {bam_sort_by_n} {bam_fn}").format(
            **locals()))
    return bam_sort_by_n


def _get_freq(name):
    """
    Check if name read contains counts (_xNumber)
    """
    try:
        counts = int(name.split("_x")[1])
    except:
        return 0
    return counts



"""Read precursor fasta file"""

from collections import defaultdict

import mirtop.libs.logger as mylog

logger = mylog.getLogger(__name__)


def read_precursor(precursor, sps=None):
    """
    Load precursor file for that species

    Args:
        *precursor(str)*: file name with fasta sequences

        *sps(str)*: if any, select species to keep.
            It'll do a `header_sequence.find(sps)`.

    Returns:
        *hairpin(dict)*: keys are precursor names and
            values are precursor sequences.
    """
    hairpin = defaultdict(str)
    name = None
    with open(precursor) as in_handle:
        for line in in_handle:
            if line.startswith(">"):
                if name in hairpin:
                    hairpin[name] = hairpin[name] + "NNNNNNNNNNNN"
                if not sps or line.find(sps) > -1:
                    name = line.strip().replace(">", " ").split()[0]
                else:
                    name = None
                logger.debug("PRECURSOR::name %s" % name)
            elif name:
                hairpin[name] += line.strip().replace("U", "T")
                logger.debug("PRECURSOR::sequence %s" % hairpin[name])
        if name:
            hairpin[name] = hairpin[name] + "NNNNNNNNNNNN"
    return hairpin



""" Read sRNAbench files"""

import os
from collections import defaultdict

import mirtop.libs.logger as mylog
from mirtop.gff.body import paste_columns, variant_with_nt
from mirtop.mirna.realign import make_cigar, make_id
from mirtop.gff.classgff import feature

logger = mylog.getLogger(__name__)


def read_file(folder, args):
    """
    Read sRNAbench file and convert to mirtop GFF format.

    Args:
        *fn(str)*: file name with sRNAbench output information.

        *database(str)*: database name.

        *args(namedtuple)*: arguments from command line.
            See *mirtop.libs.parse.add_subparser_gff()*.

    Returns:
        *reads (nested dicts)*:gff_list has the format as
            defined in *mirtop.gff.body.read()*.

    """
    reads_anno = os.path.join(folder, "reads.annotation")
    reads_iso = os.path.join(folder, "microRNAannotation.txt")
    sep = " " if args.out_format == "gtf" else "="
    sample = os.path.basename(folder)
    database = args.database
    precursors = args.precursors
    matures = args.matures

    n_out = 0
    n_in = 0
    n_ns = 0
    n_notassign = 0
    n_notindb = 0
    reads = defaultdict(dict)
    seen = set()

    source_iso = _read_iso(reads_iso)
    logger.info("Reads with isomiR information %s" % len(source_iso))
    with open(reads_anno) as handle:
        for sequence in handle:
            cols = sequence.strip().split("\t")
            query_name = cols[0]
            query_sequence = cols[0]
            if query_name not in reads and not query_sequence:
                continue
            if query_sequence and query_sequence.find("N") > -1:
                n_ns += 1
                continue
            if cols[3].find("mature") == -1:
                n_in += 1
                continue

            counts = int(cols[1])

            hit = len(set([mirna.split("#")[1] for mirna in cols[4].split("$")]))

            for nhit in cols[4].split("$"):
                logger.debug("SRNABENCH::line hit: %s" % nhit)
                hit_info = nhit.split("#")
                pos_info = hit_info[3].split(",")
                start = int(pos_info[1]) - 1
                end = start + len(query_sequence)  # int(pos_info[2]) - 1
                chrom = pos_info[0]
                mirName = hit_info[1]
                if chrom not in precursors or chrom not in matures:
                    n_notindb += 1
                if mirName not in matures[chrom]:
                    n_notindb += 1
                if (query_sequence, mirName) in seen:
                    continue

                seen.add((query_sequence, mirName))

                if (query_sequence, mirName) not in source_iso:
                    continue

                isoformat = source_iso[(query_sequence, mirName)]

                if isoformat == "mv":
                    n_notassign += 1
                    continue

                source = "isomiR" if isoformat != "NA" else "ref_miRNA"

                logger.debug("SRNABENCH::query: {query_sequence}\n"
                             "  precursor {chrom}\n"
                             "  name:  {query_name}\n"
                             "  start: {start}\n"
                             "  external: {isoformat}\n"
                             "  hit: {hit}".format(**locals()))
                logger.debug("SRNABENCH:: start %s end %s" % (start, end))
                if len(precursors[chrom]) < start + len(query_sequence):
                    n_out += 1
                    continue

                Filter = "Pass"
                cigar = make_cigar(query_sequence,
                                   precursors[chrom][start:end])
                preName = chrom
                score = "."
                strand = "+"
                idu = make_id(query_sequence)
                attrb = ("Read {query_sequence}; UID {idu}; Name {mirName};"
                         " Parent {preName}; Variant {isoformat};"
                         " Cigar {cigar}; Expression {counts};"
                         " Filter {Filter}; Hits {hit};").format(**locals())
                line = ("{chrom}\t{database}\t{source}\t{start}\t{end}\t"
                        "{score}\t{strand}\t.\t{attrb}").format(**locals())
                if args.add_extra:
                    extra = variant_with_nt(line, args.precursors,
                                            args.matures)
                    line = "%s Changes %s;" % (line, extra)

                line = paste_columns(feature(line), sep=sep)
                if start not in reads[chrom]:
                    reads[chrom][start] = []
                if Filter == "Pass":
                    n_in += 1
                    reads[chrom][start].append([idu, chrom, counts,
                                                sample, line])

    logger.info("Loaded %s reads with %s hits" % (len(reads), n_in))
    logger.info("Reads without precursor information: %s" % n_notindb)
    logger.info("Reads with MV as variant definition,"
                " not supported by GFF: %s" % n_notassign)
    logger.info("Hit Filtered by having > 3 changes: %s" % n_out)

    return reads


def _read_iso(fn):
    """
    Read definitions of isomiRs by srnabench
    """
    iso = dict()
    with open(fn) as inh:
        inh.readline()
        for line in inh:
            cols = line.strip().split("\t")
            label = cols[3].split("$")
            mirnas = cols[1].split("$")
            if len(mirnas) == 1 and len(label) > 1:
                label = [cols[3].split("$")[0]]
            if len(mirnas) != len(label):
                label = label * (len(mirnas) - len(label))
            anno = dict(zip(mirnas, label))
            logger.debug("TRANSLATE::%s with %s" % (mirnas, label))
            for m in anno:
                iso[(cols[0], m)] = _translate(anno[m], cols[4])
                logger.debug("TRANSLATE::code %s" % iso[(cols[0], m)])
    return iso


def _translate(isomirs, description):
    iso = []
    labels = isomirs.split("@")
    logger.debug("TRANSLATE::label:%s" % isomirs)
    for label in labels:
        logger.debug("TRANSLATE::label:%s" % label)
        if label == "exact":
            return "NA"
        if label.find("mv") > -1:
            return "mv"
        number_nts = label.split("|")[-1].split("#")[-1]
        if number_nts.find("-") < 0:
            number_nts = "+%s" % number_nts
        if label.find("lv3p") > -1:
            iso.append("iso_3p:%s" % number_nts)
        if label.find("lv5p") > -1:
            iso.append("iso_5p:%s" % number_nts)
        if label.find("nta") > -1:
            number_nts = label.split("|")[1].split("#")[-1]
            if number_nts.find("-") < 0:
                number_nts = "+%s" % number_nts
            iso.append("iso_add:%s" % number_nts)
        if label.find("NucVar") > -1:
            for nt in description.split(","):
                logger.debug("TRANSLATE::change:%s" % description)
                if nt == "-" or nt == "NA":
                    return "notsure"
                iso.extend(_iso_snp(int(nt.split(":")[0])))
        logger.debug("TRANSLATE::iso:%s" % iso)
    return ",".join(iso)


def _iso_snp(pos):
    iso = []
    if pos > 1 and pos < 8:
        iso.append("iso_snp_seed")
    elif pos == 8:
        iso.append("iso_snp_central_offset")
    elif pos > 8 and pos < 13:
        iso.append("iso_snp+central")
    elif pos > 12 and pos < 18:
        iso.append("iso_snp_central_supp")
    else:
        iso.append("iso_snp")
    return iso



"""Helpers to define the header fo the GFF file"""

from mirtop.gff import gff_versions as version
import mirtop.libs.logger as mylog
logger = mylog.getLogger(__name__)


def create(samples, database, custom, filter=None):
    """Create header for GFF file.

    Args:
        *samples (list)*: character list with names for samples

        *database (str)*: name of the database.

        *custom (str)*: extra lines.

        *filter (list)*: character list with filter definition.

    Returns:
        *header (str)*: header string.
    """
    header = ""
    header += _get_gff_version()
    header += _get_database(database)
    header += _get_samples(samples)
    header += custom
    return header


def _get_gff_version():
    return ("## mirGFF3. VERSION"
            " %s\n" % version.current)


def _get_samples(samples):
    return "## COLDATA: %s" % ",".join(samples)


def _get_database(database):
    if database.lower().find("mirbase") > -1:
        so = "doi:10.25504/fairsharing.hmgte8"
    elif database.lower().find("mirgenedb") > -1:
        so = "http://mirgenedb.org"
    else:
        so = "Custom."
    return ("## source-ontology: %s %s\n" % (database, so))


def _filter(filters):
    if not filters:
        return "## FILTER: PASS\n"
    return "## FILTER: %s" % ";\n".join(filters)


def read_samples(fn):
    """Read samples from the header of a GFF file.

    Args:
        *fn(str)*: GFF file to read.

    Returns:
        *(list)*: character list with sample names.
    """
    with open(fn) as inh:
        for line in inh:
            if line.startswith("## COLDATA"):
                return line.strip().split(": ")[1].strip().split(",")
    raise ValueError("%s doesn't contain COLDATA header." % fn)



"""This code helps to work with spikeins"""
import os

from collections import defaultdict
from mirtop.libs import fastq
import mirtop.libs.logger as mylog

logger = mylog.getLogger(__name__)


def convert(args):
    out_dir = args.out if args.out else os.path.dirname(args.file)
    gff_file = os.path.join(out_dir, "spikeins_pre.gff")
    fasta_file = os.path.join(out_dir, "spikeins_pre.fasta")
    spikes = read_spikeins(args.file)
    write_gff(spikes, gff_file)
    logger.info("GFF file is at %s" % gff_file)
    write_precursors(spikes, fasta_file)
    logger.info("FASTA file is at %s" % fasta_file)


def read_spikeins(in_file):
    """Read FASTA file containing small spikeins

        Args:
            *in_file(str)*: file name with sequences.

        Returns:
            *spikeins(dict)*: dictionary with keys being names
             and values being: given spike-in, synthetic precursor and
             position on the synthetic precursor.
    """
    spikeins = defaultdict(dict)
    with fastq.open_fastq(in_file) as inh:
        for line in inh:
            if line.startswith(">"):
                spike = line.strip()[1:].split()[0]
            else:
                precursor = "GGGGG" + line.strip() + "GGGGG"
                pos = [5, 4 + len(line.strip())]
                logger.debug("SPIKE::NAME::%s -> %s" % (spike, line.strip()))
                logger.debug("SPIKE::GENERATED::precursor::%s" % precursor)
                logger.debug("SPIKE::GENERATED::position::%s" % pos)
                spikeins[spike] = {'mature': line.strip(),
                                   'precursor': precursor,
                                   'position': pos}
    return spikeins


def write_precursors(spikeins, out_file):
    """
    Write FASTA file for precursors to be used to annotate isomiRs

        Args:
            *spikeins(dict)*: output from read_spikeins
            *out_file*: file name to write the FASTA information
    """
    with open(out_file, 'w') as outh:
        for spike in spikeins:
            outh.write(">pre-{0}\n{1}\n".format(spike,
                                                 spikeins[spike]['precursor']))


def write_gff(spikeins, out_file):
    """
    Write GTF file for precursors to be used to annotate isomiRs

        Args:
            *spikeins(dict)*: output from read_spikeins
            *out_file*: file name to write the GFF information
    """
    with open(out_file, 'w') as outh:
        outh.write("#microRNAs:               spikeins_v1\n")
        for spike in spikeins:
            outh.write("\t".join(["chr%s" % spike, ".",
                                  "miRNA_primary_transcript",
                                  "1",
                                  str(len(spikeins[spike]['precursor'])),
                                  ".", "+", ".",
                                  "Name=pre-%s;ID=pre-%s" % (spike, spike),
                                  "\n"]))
            outh.write("\t".join(["chr%s" % spike, ".", "miRNA",
                                  str(spikeins[spike]['position'][0] + 1),
                                  str(spikeins[spike]['position'][1] + 1),
                                  ".", "+", ".",
                                  "Derives_from=pre-%s;"
                                  "Name=%s;ID=%s;" % (spike, spike, spike),
                                  "\n"]))



"""
Compare multiple GFF files to a reference
"""

from __future__ import print_function

import os

from mirtop.gff.classgff import feature
from mirtop.mirna.realign import read_id
import mirtop.libs.logger as mylog

logger = mylog.getLogger(__name__)


def compare(args):
    """
    From a list of GFF files produce comparison with a reference set.

    Args:
        *args(namedtuple)*: arguments parsed from command line with
            *mirtop.libs.parse.add_subparser_compare()*.
            First file will be considered the reference set.

    Returns:
        *(out_file)*: comparison of the GFF files with the reference.
    """
    result = dict()
    reference = read_reference(args.files[0])
    for fn in args.files[1:]:
        if not os.path.exists(fn):
            raise IOError("%s doesn't exist" % fn)
        result[os.path.basename(fn)] = _compare_to_reference(fn, reference)
    if args.out != "tmp_mirtop":
        fn_out = os.path.join(args.out, "summary.txt")
        with open(fn_out, 'w') as outh:
            for fn in result:
                print("sample\tidu\tseq\ttag\tsame_mirna\t%s" % "\t".join(result[fn][0][3].keys()), file=outh)
                for line in result[fn]:
                    read = read_id(line[0])
                    acc = "\t".join([line[3][v] for v in line[3]])
                    print("%s\t%s\t%s\t%s\t%s\t%s" % (fn, line[0], read, line[1], line[2], acc), file=outh)


def read_reference(fn):
    """Read GFF into UID:Variant

    Args:
        *fn (str)*: GFF file.

    Returns:
        *srna (dict)*: dict with >>> {'UID': 'iso_snp:-2,...'}
    """
    srna = dict()
    with open(fn) as inh:
        for line in inh:
            if line.startswith("#"):
                continue
            gff = feature(line)
            attr = gff.attributes
            srna[attr['UID']] = [_simplify(attr['Variant']), attr]
    return srna


def _compare_to_reference(fn, reference):
    same = 0
    diff = list()
    extra = list()
    miss = list()
    results = list()
    seen = 0
    seen_reference = set()
    with open(fn) as inh:
        for line in inh:
            if line.startswith("#"):
                continue
            gff = feature(line)
            attr = gff.attributes
            if attr['UID'] in reference:
                mirna = "Y" if attr['Name'] == reference[attr['UID']][1]['Name'] else attr['Name']
                accuracy = _accuracy(_simplify(attr['Variant']), reference[attr['UID']][0])
                results.append([attr['UID'], "D", mirna, accuracy])
                if _simplify(attr['Variant']) == reference[attr['UID']][0]:
                    same += 1
                else:
                    diff.append("%s | reference: %s" % (line.strip(), reference[attr['UID']][1]))
                seen += 1
                seen_reference.add(attr['UID'])
            else:
                extra.append("%s | extra" % line.strip())
                results.append([attr['UID'], "E", attr['Name'], _accuracy(_simplify(attr['Variant']), "")])
    for uid in reference:
        if uid not in seen_reference:
            results.append([uid, "M", "N", _accuracy("", reference[uid][0])])
            miss.append("| miss %s" % reference[uid][1])
    logger.info("Number of sequences found in reference: %s" % seen)
    logger.info("Number of sequences matches reference: %s" % same)
    logger.info("Number of sequences different than reference: %s" % len(diff))
    logger.info("Number of sequences extra sequences: %s" % len(extra))
    logger.info("Number of sequences missed sequences: %s" % len(miss))
    return results


def _simplify(variant):
    simple = [v.split(":")[0] for v in variant.split(",")]
    return ",".join(simple)


def _get_samples(fn):
    with open(fn) as inh:
        for line in inh:
            if line.startswith("## COLDATA"):
                return line.strip().split(": ")[1].strip().split(",")
    raise ValueError("%s doesn't contain COLDATA header." % fn)


def _accuracy(target, reference):
    """Compare each isomir label in Variant field
       and return a list with values whether:
           FP: no in reference
           FN: no in target
           TP: same values
    """
    logger.debug("COMPARE::ACCURACY::values %s vs %s" % (target, reference))
    accuracy = dict()
    types = ["iso_5p", "iso_3p", "iso_add", "iso_snp",
             "iso_snp_seed", "iso_snp_central",
             "iso_snp_central_supp", "iso_snp_central_offset"]
    for t in types:
        if t in reference:
            accuracy[t] = "TP" if t in target else "FN"
        else:
            accuracy[t] = "TN" if t not in target else "FP"
    logger.debug("COMPARE::ACCURACY::%s" % accuracy.keys())
    logger.debug("COMPARE::ACCURACY::%s" % accuracy.values())
    return accuracy



from mirtop.gff.classgff import feature
import mirtop.libs.logger as mylog
from mirtop.gff import gff_versions as version
from mirtop.mirna.realign import make_id

logger = mylog.getLogger(__name__)


def _check_header(header):
    """ Check that header has the minimum mandatory fields
    """
    # Check mandatory fields present:
    num_samples = 0
    matching = []
    mandatory_fields = ["source-ontology", "COLDATA", "VERSION"]
    for field in mandatory_fields:
            match = ([s for s in header if field in s])
            if len(match) != 0:
                matching.append(match)

    all_present = len(matching) == len(mandatory_fields)
    if all_present:
        for line in header:
            if line.startswith("## COLDATA"):
                samples = line.strip().split(": ")[1].strip().split(",")
                num_samples = len(samples)
    return [all_present, num_samples]


def _check_line(line, num, num_samples):
    """ Check file for minimum
    """
    gff = feature(line)
    fields = gff.columns
    attr = gff.attributes

    # Check seqID
    if not fields['chrom']:
        logger.error('MISSING seqID in line %s' % (num))

    # Check source
    source = (fields['source']).lower()
    valid_source = False
    valid_sources = ["mirBase", "mirgeneDB"]
    if (any(s.lower() in source for s in valid_sources)):
        valid_source = True

    if valid_source is False:
        logger.error('INCORRECT SOURCE in line %s' % (num))

    # Check type
    type = fields['type']

    source = (fields['source']).lower()
    valid_type = False
    if type in ["ref_miRNA", "isomiR"]:
        valid_type = True

    if valid_type is False:
        logger.error('INCORRECT TYPE in line %s' % (num))

    # Check start/end
    if not fields['start']:
        logger.error('MISSING START value in line %s' % (num))

    if not fields['end']:
        logger.error('MISSING END value in line %s' % (num))

    # Check strand
    if str(fields['strand']) not in ["+", "-"]:
        logger.error('INCORRECT STRAND in line %s' % (num))

    # Check UID
    if 'UID' not in attr:
        logger.error('UID not found in line %s' % (num))
    else:
        if not make_id(attr['UID']):
            logger.error('UID is not in a correct format in line %s. '
                         'Use mirtop gff to fix this or open an issue.' % num)

    # Check attribute-variant
    variant = (attr['Variant']).lower()
    valid_variant = False
    valid_variants = version.GFFv[version.current]
    if (any(s.lower() in variant for s in valid_variants)):
        valid_variant = True

    if valid_variant is False:
        logger.error('INCORRECT VARIANT type in line %s' % (num))

    # Check attribute-expression

    expression = attr['Expression'].strip().split(",")
    expression = filter(None, expression)
    if len(expression) != num_samples:
        logger.error('INCORRECT number of EXPRESSION VALUES \
        in line %s' % (num))


def _check_file(file):
    """
    """
    # Get header to check.
    header = []
    with open(file) as ch:
        for line in ch:
            if line.startswith("##"):
                header.append(line)
            else:
                break
    all_present, num_samples = _check_header(header)
    if all_present is False:
        logger.error("%s doesn't contain all \
        the mandatory fields for the header." % file)
    logger.info("HEADER CHECKED")
    # Check lines

    with open(file) as ch:
        for num, line in enumerate(ch, 1):
            if line.startswith("##"):
                next
            else:
                _check_line(line, num, num_samples)


def check_multiple(args):
    for file in args.files:
        _check_file(file)
        logger.info('%s checked' % file)
